/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    Hash Table Management Module version 1.0

    Copyright (C) 2002 by Sundaresh Venugopal

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    You may contact the author via email at one of the following address
    sundaresh@mail.com , or  cbuff@programmer.net .

~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/

#ifndef HASH_H
#define HASH_H

void memswap(void *loc0,void *loc1,int size);

#define resize(size,count,ops) ((size) / (ops) * (count))
#define _resize(size,count,ops) ((size) / (ops) * ((count) << (1)))

typedef int (* hash_fn)(unsigned long key,int size);
typedef int (* match_fn)(void *element);
typedef unsigned long (* key_fn)(void *element);
typedef int (* fail_fn)(void *element);

#define EHRESIZE_FAIL (-1923)

extern int hashes;

void * hnew(int *about,int width,float occupancy);
void hdispose(void *base);
void *hsearch(void *base,int width,int size,unsigned long key,hash_fn hash,match_fn match);
void *_hsearch(void *base,int width,int size,unsigned long key,hash_fn hash,match_fn match);
void **hresize(void **base,int width,int *size,int newsize,key_fn key,hash_fn hash,match_fn match,fail_fn fail);
void **_hresize(void **base,int width,int *size,int newsize,key_fn key,hash_fn hash,match_fn match,fail_fn fail);

typedef void (* freee_fn)(void *element);

struct syn {
        void *element;
        struct syn *next;
};
typedef struct syn *syn_t;

struct shptr {
       int index;
       struct syn *prev, *curr;
};
typedef struct shptr *shptr_t;

syn_t * shnew(int *size);
void shdispose(syn_t *base,int size,freee_fn freee);
#define SHSEEK_FIRST (0)
#define SHSEEK_NEXT (1)
#define SHSEEK_LAST (2)
shptr_t shseek(syn_t *base,int size,int seek,shptr_t ptr);
shptr_t shsearch(syn_t *base,int size,unsigned long key,hash_fn hash,match_fn match,shptr_t ptr);
shptr_t _shsearch(syn_t *base,int size,unsigned long key,hash_fn hash,match_fn match,shptr_t ptr);
syn_t **shresize(syn_t **base,int *size,int newsize,key_fn key,hash_fn hash);
syn_t **_shresize(syn_t **base,int *size,int newsize,key_fn key,hash_fn hash);

#endif
